-- File ECOA-File.ads

-- Standard ECOA Types
with ECOA;


package ECOA.File is

-- Record type
type Reference is
   record
      descriptor : ECOA.Signed_32_Type;
   end record;

-- Fixed Array type
type FileName_Index is range 0..256-1;
type FileName is array
    (FileName_Index) of ECOA.Character_8_Type;

-- Enum type
type FileFlag is 
   (FileFlag_READ,
FileFlag_WRITE,
FileFlag_READWRITE,
FileFlag_APPEND,
FileFlag_NONBLOCK,
FileFlag_CREATE,
FileFlag_TRUNCATE,
FileFlag_EXCLUSIVE,
FileFlag_NOCTTY,
FileFlag_PRIVATE,
FileFlag_LARGEFILE,
FileFlag_BINARY);
for FileFlag use
   (FileFlag_READ => 0,
FileFlag_WRITE => 1,
FileFlag_READWRITE => 2,
FileFlag_APPEND => 8,
FileFlag_NONBLOCK => 128,
FileFlag_CREATE => 256,
FileFlag_TRUNCATE => 512,
FileFlag_EXCLUSIVE => 1024,
FileFlag_NOCTTY => 2048,
FileFlag_PRIVATE => 4096,
FileFlag_LARGEFILE => 8192,
FileFlag_BINARY => 16384);
for FileFlag'Size use ECOA.Unsigned_16_Type'Size;
for FileFlag'Alignment use ECOA.Unsigned_16_Type'Alignment;

-- Enum type
type SeekBase is 
   (SeekBase_SEEK_SET,
SeekBase_SEEK_CUR,
SeekBase_SEEK_END);
for SeekBase use
   (SeekBase_SEEK_SET => 0,
SeekBase_SEEK_CUR => 1,
SeekBase_SEEK_END => 2);
for SeekBase'Size use ECOA.Unsigned_8_Type'Size;
for SeekBase'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Enum type
type Permission is 
   (Permission_READ,
Permission_WRITE,
Permission_EXECUTE,
Permission_READEXEC,
Permission_ALL);
for Permission use
   (Permission_READ => 4,
Permission_WRITE => 2,
Permission_EXECUTE => 1,
Permission_READEXEC => 5,
Permission_ALL => 7);
for Permission'Size use ECOA.Unsigned_8_Type'Size;
for Permission'Alignment use ECOA.Unsigned_8_Type'Alignment;

-- Record type
type Permissions is
   record
      Owner : Permission;
      Group : Permission;
      Others : Permission;
   end record;

-- Record type
type FileSpec is
   record
      FileName : FileName;
      Access : FileFlag;
      CreatePermissions : Permissions;
   end record;


end ECOA.File;
