/*
 * ECOA.File.API.c
 *
 * Experimental file API for ECOA.
 *
 * This implementation is a thin bind onto POSIX...
 *
 *  Created on: 21 Apr 2015
 *      Author: SW_APPLEBY
 */

#include "ECOA.h"
#include "ECOA__File.h"
#include "fcntl.h"
#include "unistd.h"

#define Pub_Main_Im__context void

ECOA__return_status Pub_Main_Im_container__open(
						Pub_Main_Im__context   *context,
						ECOA__File__FileName    path,
						ECOA__File__Reference  *fileRef,
						ECOA__File__FileFlag    flags,
						ECOA__File__Permissions perms )
{
	int   fileFlags = O_RDONLY, fileMode = 0;
	//
	// Sort out file flags.
	// We CANNOT assume that our numbering matches the toolchain/operating system...
	if( flags & ECOA__File__FileFlag_WRITE )     fileFlags |= O_WRONLY;
	if( flags & ECOA__File__FileFlag_READWRITE ) fileFlags |= O_RDWR;
	if( flags & ECOA__File__FileFlag_APPEND )    fileFlags |= O_APPEND;
	if( flags & ECOA__File__FileFlag_NONBLOCK )  fileFlags |= O_NONBLOCK;
	if( flags & ECOA__File__FileFlag_CREATE )    fileFlags |= O_CREAT;
	if( flags & ECOA__File__FileFlag_TRUNCATE )  fileFlags |= O_TRUNC;
	if( flags & ECOA__File__FileFlag_EXCLUSIVE ) fileFlags |= O_EXCL;
	if( flags & ECOA__File__FileFlag_NOCTTY )    fileFlags |= O_NOCTTY;
#ifdef O_PRIV
	if( flags & ECOA__File__FileFlag_PRIVATE )   fileFlags |= O_PRIV;
#endif
#ifdef O_LARGEFILE
	if( flags & ECOA__File__FileFlag_LARGEFILE ) fileFlags |= O_LARGEFILE;
#endif
#ifdef O_BINARY
	if( flags & ECOA__File__FileFlag_BINARY )    fileFlags |= O_BINARY;
#endif
	//
	// Sort out mode (permission) bits.
	// We CANNOT assume that our numbering matches the toolchain/operating system...
#ifdef S_IROTH
	if( perms.Others & ECOA__File__Permission_READ )    fileMode |= S_IROTH;
	if( perms.Others & ECOA__File__Permission_WRITE )   fileMode |= S_IWOTH;
	if( perms.Others & ECOA__File__Permission_EXECUTE ) fileMode |= S_IXOTH;
	if( perms.Group  & ECOA__File__Permission_READ )    fileMode |= S_IRGRP;
	if( perms.Group  & ECOA__File__Permission_WRITE )   fileMode |= S_IWGRP;
	if( perms.Group  & ECOA__File__Permission_EXECUTE ) fileMode |= S_IXGRP;
	if( perms.Owner  & ECOA__File__Permission_READ )    fileMode |= S_IRUSR;
	if( perms.Owner  & ECOA__File__Permission_WRITE )   fileMode |= S_IWUSR;
	if( perms.Owner  & ECOA__File__Permission_EXECUTE ) fileMode |= S_IXUSR;
#else
	if( perms.Others & ECOA__File__Permission_READ )    fileMode |= 0004;
	if( perms.Others & ECOA__File__Permission_WRITE )   fileMode |= 0002;
	if( perms.Others & ECOA__File__Permission_EXECUTE ) fileMode |= 0001;
	if( perms.Group  & ECOA__File__Permission_READ )    fileMode |= 0040;
	if( perms.Group  & ECOA__File__Permission_WRITE )   fileMode |= 0020;
	if( perms.Group  & ECOA__File__Permission_EXECUTE ) fileMode |= 0010;
	if( perms.Owner  & ECOA__File__Permission_READ )    fileMode |= 0400;
	if( perms.Owner  & ECOA__File__Permission_WRITE )   fileMode |= 0200;
	if( perms.Owner  & ECOA__File__Permission_EXECUTE ) fileMode |= 0100;
#endif
	//
	if(( fileRef->descriptor = open( path, fileFlags, fileMode )) < 0 ){
		return ECOA__return_status_INVALID_IDENTIFIER;
	}else
		return ECOA__return_status_OK;
}

ECOA__return_status Pub_Main_Im_container__close(
						Pub_Main_Im__context  *context,
						ECOA__File__Reference *fileRef )
{
	ECOA__return_status sts = ECOA__return_status_OK;
	if( fileRef ){
		if( close( fileRef->descriptor ) < 0 )
			sts = ECOA__return_status_RESOURCE_NOT_AVAILABLE;
		else
			sts = ECOA__return_status_OK;
	}else
		sts = ECOA__return_status_INVALID_IDENTIFIER;
	return sts;
}

ECOA__return_status Pub_Main_Im_container__read(
						Pub_Main_Im__context  *context,
						ECOA__File__Reference *fileRef,
						ECOA__byte            *buffer,
						ECOA__uint64           byteCount,
						ECOA__uint64          *bytesRead )
{
	if(( *bytesRead = (ECOA__uint64)read( fileRef->descriptor, (void*)buffer, (size_t)byteCount )) < 0 )
		return ECOA__return_status_INVALID_IDENTIFIER;
	else
	if( *bytesRead == 0 )
		return ECOA__return_status_NO_DATA;
	else
		return ECOA__return_status_OK;
}

ECOA__return_status Pub_Main_Im_container__write(
						Pub_Main_Im__context  *context,
						ECOA__File__Reference *fileRef,
						ECOA__byte            *buffer,
						ECOA__uint64           byteCount,
						ECOA__uint64          *bytesWritten )
{
	if(( *bytesWritten = (ECOA__uint64)write( fileRef->descriptor, (void*)buffer, (size_t)byteCount )) < 0 )
		return ECOA__return_status_INVALID_IDENTIFIER;
	else
		return ECOA__return_status_OK;
}

ECOA__return_status Pub_Main_Im_container__seek(
						Pub_Main_Im__context  *context,
						ECOA__File__Reference *fileRef,
						ECOA__int64            seekPosn,
						ECOA__File__SeekBase   base,
						ECOA__int64           *actualPosn )
{
	int whence;
	//
	// Sortout the seek base.
	// We CANNOT assume that our numbering matches the toolchain/operating system...
	if( base == ECOA__File__SeekBase_SEEK_SET ) whence = SEEK_SET;
	if( base == ECOA__File__SeekBase_SEEK_CUR ) whence = SEEK_CUR;
	if( base == ECOA__File__SeekBase_SEEK_END ) whence = SEEK_END;
	//
	if(( *actualPosn = (ECOA__int64)lseek( fileRef->descriptor, (off_t)seekPosn, whence )) < 0 )
		return ECOA__return_status_INVALID_PARAMETER;
	else
		return ECOA__return_status_OK;
}

ECOA__return_status Pub_Main_Im_container__delete(
						Pub_Main_Im__context *context,
						ECOA__File__FileName  path )
{
	if( unlink( path ) < 0 )
		return ECOA__return_status_INVALID_IDENTIFIER;
	else
		return ECOA__return_status_OK;
}

